{$I-}
unit IniEx;

interface

uses
   SysUtils, Forms, IniFiles, Classes;

const
   ini = '.INI';

var
  //         .INI
   NoErrorMessage : boolean = TRUE;
  //    
   SectionName    : string  = 'Settings';
  //   . .  .
   FileName       : string;

//       { XX XX XX XX }
//    - 16-    
//   - 16384 .
function DataToDataStr(const Data; SizeOfData : integer) : string;
//    {     }  
//       
function DataStrToData(const DataStr : string; var Data; SizeOfData : integer) : integer;

function StrToBool(s : string): boolean;
function BoolToStr(b : boolean) : string;

//   Write*  TRUE   

function ReadString(const Ident,Default : string) : string;
function WriteString(const Ident,Value : string) : boolean;

function ReadBool(const Ident : string; Default : boolean) : boolean;
function WriteBool(const Ident : string; const Value : boolean) : boolean;

function ReadFloat(const Ident : string; Default : extended) : extended;
function WriteFloat(const Ident : string; const Value : extended) : boolean;

function ReadCurrency(const Ident : string; Default : currency) : currency;
function WriteCurrency(const Ident : string; const Value : currency) : boolean;

function ReadOrdinal(const Ident : string; Default : variant) : variant;
function WriteOrdinal(const Ident : string; const Value : variant) : boolean;

function ReadInteger(const Ident : string; Default : longint) : longint;
function WriteInteger(const Ident : string; const Value : longint) : boolean;

function ReadDateTime(const Ident : string; Default : TDateTime) : TDateTime;
function WriteDateTime(const Ident : string; const Value : TDateTime) : boolean;

function ReadDate(const Ident : string; Default : TDateTime) : TDateTime;
function WriteDate(const Ident : string; const Value : TDateTime) : boolean;

function ReadTime(const Ident : string; Default : TDateTime) : TDateTime;
function WriteTime(const Ident : string; const Value : TDateTime) : boolean;

function ReadBinaryData(const Ident : string; var Data; SizeOfData : integer) : integer;
function WriteBinaryData(const Ident : string; const Data; SizeOfData : integer) : boolean;

function EraseSection(const Section : string) : boolean;
function EraseKey(const Ident: string) : boolean;

function ReadSections(Sections : TStrings) : boolean;

implementation

const
  gMAX_DATA_LENGTH = 16384;
  HEXDIGITS = ['0'..'9','A'..'F'];
  SYMBOLS   = [' ','0'..'9','A'..'F','}'];

type
  TDataArray = array[1..gMAX_DATA_LENGTH] of byte;
  PDataArray = ^TDataArray;

function ByteToHexStr(b : byte) : string;
begin
  SetLength(Result,3);
  Result[1] := Chr(b shr 4 + Ord('0'));
  if Result[1] > '9' then Inc(Result[1],Ord('A')-Ord('9')-1);
  Result[2] := Chr(b and $0F + Ord('0'));
  if Result[2] > '9' then Inc(Result[2],Ord('A')-Ord('9')-1);
  Result[3] := ' ';
end;

function HexStrToByte(s : string) : byte;
var
  b : byte;
begin
  b := Ord(s[1]) - Ord('0');
  if b > 9 then Dec(b,Ord('A')-Ord('9')-1);
  Result := b shl 4;
  b := Ord(s[2]) - Ord('0');
  if b > 9 then Dec(b,Ord('A')-Ord('9')-1);
  Result := Result or b;
end;

function DataToDataStr;
var
  i : integer;
  p : PDataArray;
begin
  p := @Data;
  Result := '{ ';
  i := gMAX_DATA_LENGTH;
  if SizeOfData < i then i := SizeOfData;
  for i := 1 to i do Result := Result + ByteToHexStr(p^[i]);
  Result := Result + '}';
end;

function DataStrToData;
var
  i : integer;
  s : (sNone,sBegin,sSpace,sByte,sEnd);
  p : PDataArray;
begin
  s := sNone;
  Result := 0;
  if Length(DataStr) < 4 then Exit; // {XX}    
  p := @Data;
  i := 0;
  repeat Inc(i);
    case DataStr[i] of
      ' ' : if (s = sBegin) or (s = sByte) then s := sSpace;
      '{' : if s = sNone then s := sBegin else s := sEnd;
      '0'..'9','A'..'F' :
        if (s = sBegin) or (s = sSpace) then begin
          s := sByte;
          if (DataStr[i+1] in HEXDIGITS) and (DataStr[i+2] in SYMBOLS) then begin
            Inc(Result);
            p^[Result] := HexStrToByte(Copy(DataStr,i,2));
            Inc(i);
          end; // if
        end; // if
      else s := sEnd;
    end; // case
  until (s = sEnd) or (i >= Length(DataStr)) or (Result >= SizeOfData);
end;

procedure NoSave;
begin
  if NoErrorMessage then Exit;
  Application.MessageBox(PChar('    ''' + FileName +'''.'),'',0);
end;

function StrToBool;
begin
  s := AnsiUpperCase(Trim(s));
  Result := FALSE;
  if (s = '1') or (s = 'Y') or (s = '+') or (s = '') then Result := TRUE;
  if (s = '') or (s = 'YES') or (s = 'TRUE') then Result := TRUE;
end;

function BoolToStr;
begin
  if b then Result := 'Y' else Result := 'N';
end;

function SignifyStr(IniFile : TIniFile; const Ident,Default : string) : string;
var
  p : integer;
  s : string;
begin
  s := IniFile.ReadString(SectionName,Ident,Default);
  p := Pos(';',s);
  if p < 2 then Result := Trim(s) else Result := Trim(Copy(s,1,p-1));
end;

function Comment(IniFile : TIniFile; const Ident : string) : string;
var
  p : integer;
  s : string;
begin
  s := IniFile.ReadString(SectionName,Ident,'');
  p := Pos(';',s);
  if p < 1 then Result := '' else Result := ' ' + Trim(Copy(s,p,Length(s)-p+1));
end;

function ReadString;
var
  f : TIniFile;
begin
  f := TIniFile.Create(FileName);
  with f do
    try
      Result := SignifyStr(f,Ident,Default);
    finally
      Free;
    end;
end;

function WriteString;
var
  f : TIniFile;
begin
  Result := TRUE;
  f := TIniFile.Create(FileName);
  with f do
    try
      WriteString(SectionName,Ident,Value + Comment(f,Ident));
      Free;
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadBool;
var
  f : TIniFile;
begin
  f := TIniFile.Create(FileName);
  with f do
    try
      Result := StrToBool(SignifyStr(f,Ident,BoolToStr(Default)));
    finally
      Free;
    end;
end;

function WriteBool;
var
  f : TIniFile;
begin
  Result := TRUE;
  f := TIniFile.Create(FileName);
  with f do
    try
      WriteString(SectionName,Ident,BoolToStr(Value) + Comment(f,Ident));
      Free
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadFloat;
var
  f : TIniFile;
begin
  f := TIniFile.Create(FileName);
  with f do
    try
      Result := StrToFloat(SignifyStr(f,Ident,FloatToStr(Default)));
    finally
      Free;
    end;
end;

function WriteFloat;
var
  f : TIniFile;
begin
  Result := TRUE;
  f := TIniFile.Create(FileName);
  with f do
    try
      WriteString(SectionName,Ident,FloatToStr(Value) + Comment(f,Ident));
      Free;
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadOrdinal;
var
  f : TIniFile;
begin
  f := TIniFile.Create(FileName);
  with f do
    try
      Result := StrToInt(SignifyStr(f,Ident,IntToStr(Default)));
    finally
      Free;
    end;
end;

function WriteOrdinal;
var
  f : TIniFile;
begin
  Result := TRUE;
  f := TIniFile.Create(FileName);
  with f do
    try
      WriteString(SectionName,Ident,IntToStr(Value) + Comment(f,Ident));
      Free
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadCurrency;
var
  f : TIniFile;
begin
  f := TIniFile.Create(FileName);
  with f do
    try
      Result := StrToCurr(SignifyStr(f,Ident,CurrToStr(Default)));
    finally
      Free;
    end;
end;

function WriteCurrency;
var
  f : TIniFile;
begin
  Result := TRUE;
  f := TIniFile.Create(FileName);
  with f do
    try
      WriteString(SectionName,Ident,CurrToStr(Value) + Comment(f,Ident));
      Free
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadInteger;  begin Result := ReadOrdinal(Ident,Default) end;
function WriteInteger; begin Result := WriteOrdinal(Ident,Value) end;

function ReadDateTime;  begin Result := ReadFloat(Ident,Default) end;
function WriteDateTime; begin Result := WriteFloat(Ident,Value) end;

function ReadDate;  begin Result := ReadFloat(Ident,Default) end;
function WriteDate; begin Result := WriteFloat(Ident,Value) end;

function ReadTime;  begin Result := ReadFloat(Ident,Default) end;
function WriteTime; begin Result := WriteFloat(Ident,Value) end;

function ReadBinaryData;  begin Result := DataStrToData(ReadString(Ident,''),Data,SizeOfData) end;
function WriteBinaryData; begin Result := WriteString(Ident,DataToDataStr(Data,SizeOfData)) end;

function EraseSection;
begin
  Result := TRUE;
  with TIniFile.Create(FileName) do
    try
      EraseSection(Section);
      Free;
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function EraseKey;
begin
  Result := TRUE;
  with TIniFile.Create(FileName) do
    try
      DeleteKey(SectionName,Ident);
      Free;
    except
      Result := FALSE;
      Free;
      NoSave;
    end;
end;

function ReadSections;
begin
  Result := TRUE;
  with TIniFile.Create(FileName) do
    try
      ReadSections(Sections);
      Free;
    except
      Result := FALSE;
      Free;
    end;
end;

begin
  FileName := ChangeFileExt(Application.ExeName,ini)
end.


